/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
package com.ebmwebsourcing.easycommons.uuid;

import org.junit.Assert;
import org.junit.Test;

import com.ebmwebsourcing.easycommons.uuid.framework.orchestrator.GenUUIDFromMultipleQualifiedGeneratorOrchestrator;
import com.ebmwebsourcing.easycommons.uuid.framework.orchestrator.GenUUIDFromSingleQualifiedGeneratorOrchestrator;

public class QualifiedUUIDGeneratorTest {

    /**
     * Number of simultaneously launched uuid generator thread
     */
    private final int POOL_SIZE = 100;

    /**
     * Number of uuids generated by each thread
     */
    private final int ITERATIONS = 100;

    @Test
    public void testGetNewID() {
        String domain = "foo";
        QualifiedUUIDGenerator generator = new QualifiedUUIDGenerator(domain);
        String uuid = generator.getNewID();
        String uuidPatern = "^" + domain
                + ":[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}$";
        Assert.assertTrue("Generated uuid must match the followwing pattern "
                + "'domain:[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}'",
                uuid.matches(uuidPatern));
    }

    @Test
    public void testGetUuidMaxLength() {
        String domain = "foo";
        QualifiedUUIDGenerator generator = new QualifiedUUIDGenerator(domain);
        int uuidLength = generator.getUuidMaxLength();
        String uuid = generator.getNewID();
        Assert.assertEquals(uuid.length(), uuidLength);
    }

    @Test
    public void testGenerateUniqueID() {
        try {
            System.gc();
            new GenUUIDFromMultipleQualifiedGeneratorOrchestrator().orchestrate(POOL_SIZE,
                    ITERATIONS);
        } catch (Exception e) {
            Assert.fail(e.getMessage());
        }
    }

    @Test
    public void testGetNewIDMethodIsThreadsafe() {
        try {
            System.gc();
            new GenUUIDFromSingleQualifiedGeneratorOrchestrator()
                    .orchestrate(POOL_SIZE, ITERATIONS);
        } catch (Exception e) {
            Assert.fail(e.getMessage());
        }
    }

}
